package net.nigreon.blegps

import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.app.Service
import android.content.*
import android.location.LocationManager
import android.os.*
import android.support.v4.app.NotificationCompat
import android.support.v4.content.LocalBroadcastManager
import android.support.v7.app.AppCompatActivity
import android.util.Log
import java.io.File
import java.io.FileNotFoundException
import java.io.FileOutputStream
import java.io.IOException
import java.nio.ByteBuffer
import java.nio.ByteOrder
import java.text.SimpleDateFormat
import java.util.*
import kotlin.experimental.or


class BLEMockLocationService : Service() {
    private val LOG_TAG = "BLEMockLocationService"
    private val CHANNEL_ID = "NotifID"
    public val ACTIVATION_CHANGE = 1
    private lateinit var pBLE: BLEProvider
    private lateinit var mockGPS: MockLocationProvider
    private var locationReceived = false
    private var qualityReceived = false
    private var tempenable = false
    private lateinit var gpxoutputstream: FileOutputStream

    private lateinit var sharedPreferences: SharedPreferences

    var gpsConf: GPSConfiguration = GPSConfiguration()

    // Location
    private var currentLatitude: Double = -1.0
    private var currentLongitude: Double = -1.0
    private var currentElevation: Double = -1.0
    private var currentSpeed: Float = -1.0f
    private var currentHeading: Float = -1.0f
    private var currentEHPE: Float = -1.0f
    // Quality
    private var currentEVPE: Float = -1.0f
    private var currentHDOP: Float = -1.0f
    private var currentVDOP: Float = -1.0f
    private var currentElevationGPS: Double = -1.0

    private var currentTemperature: Float = 0.0f

    private val binder = sBinder()

    fun locationChanged(intent: Intent)
    {
        Log.v(LOG_TAG, "Latitude Service Broadcast received")
        currentLatitude = intent.getDoubleExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONLAT,-1.0)
        currentLongitude = intent.getDoubleExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONLON,-1.0)
        currentEHPE = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONEHPE,-1.0f)
        currentElevation = intent.getDoubleExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONELE,-1.0)
        currentSpeed = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONSPEED,-1.0f)
        currentHeading = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYLOCATIONHEADING,-1.0f)
        mockGPS.pushLocation(currentLatitude,currentLongitude,currentElevation, currentSpeed, currentHeading, currentEHPE)
        locationReceived = true
    }

    fun qualityChanged(intent: Intent)
    {
        //Log.v(LOG_TAG, "Latitude Service Broadcast received")
        currentEHPE = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYEHPE,-1.0f)
        currentEVPE = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYEVPE,-1.0f)
        currentHDOP = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYHDOP,-1.0f)
        currentVDOP = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYVDOP,-1.0f)
        currentElevationGPS = intent.getDoubleExtra(ServicesConstants.BROADCAST_KEY.KEYQUALITYELEGPS,-1.0)
        qualityReceived = true
    }

    fun bmpStatusChanged(intent: Intent)
    {
        tempenable = intent.getBooleanExtra(ServicesConstants.BROADCAST_KEY.KEYBMPSTATUS, false)
    }

    fun tempChanged(intent: Intent)
    {
        currentTemperature = intent.getFloatExtra(ServicesConstants.BROADCAST_KEY.KEYTEMP, -1.0f)
    }

    fun taskerChanged(bundle: Bundle) {
        Log.v(LOG_TAG, "taskerChanged")
        if(bundle.containsKey(ServicesConstants.BROADCAST_KEY.KEYBOOST)) {
            if (bundle.getBoolean(ServicesConstants.BROADCAST_KEY.KEYBOOST)) {
                //gpsConf.gpsBoost = intent.getBooleanExtra(ServicesConstants.BROADCAST_KEY.KEYBOOST, false)
                sendConfBLE(2)
            } else {
                sendConfBLE(1)
            }
            //gpsConf.gpsBoost = bundle.getBoolean(ServicesConstants.BROADCAST_KEY.KEYBOOST)

            //Log.v(LOG_TAG, "Received Tasker Intent : Boost ${gpsConf.gpsBoost}")
        }
    }

    override fun onCreate() {
        super.onCreate()

        sharedPreferences = baseContext.getSharedPreferences("blegps_config",
            AppCompatActivity.MODE_PRIVATE
        )

        mockGPS = MockLocationProvider(LocationManager.GPS_PROVIDER, baseContext)

        val brReceiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context?, intent: Intent?) {
                when (intent?.action) {
                    ServicesConstants.BROADCAST_FILTER.FILTERLOCATION -> locationChanged(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERQUALITY-> qualityChanged(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERTEMP-> tempChanged(intent)
                    ServicesConstants.BROADCAST_FILTER.FILTERBMP-> bmpStatusChanged(intent)


                    //BROADCAST_CHANGE_TYPE_CHANGED -> handleChangeTypeChanged()
                }
            }
        }

        val manager = LocalBroadcastManager.getInstance(this)
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERLOCATION))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERQUALITY))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERTEMP))
        manager.registerReceiver(brReceiver, IntentFilter(ServicesConstants.BROADCAST_FILTER.FILTERBMP))
        //pBLE.registerBMP()
    }

    override fun onStartCommand(intent: Intent, flags: Int, startId: Int): Int {
        if(intent.action == ServicesConstants.MOCKSERVICEACTION.STARTFOREGROUND_ACTION) {
            Log.i(LOG_TAG, "Received Start Foreground Intent ")

            val startIntent = Intent(this, MainActivity::class.java)
            startIntent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT)
            val contentIntent = PendingIntent.getActivity(this, 0, startIntent,
                PendingIntent.FLAG_UPDATE_CURRENT)


            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                // Create the NotificationChannel
                val name = "ChannelName"
                val descriptionText = "ChannelDescription"
                val importance = NotificationManager.IMPORTANCE_DEFAULT
                val mChannel = NotificationChannel(CHANNEL_ID, name, importance)
                mChannel.description = descriptionText
                // Register the channel with the system; you can't change the importance
                // or other notification behaviors after this
                val notificationManager = getSystemService(NOTIFICATION_SERVICE) as NotificationManager
                notificationManager.createNotificationChannel(mChannel)
            }

            var notification = NotificationCompat.Builder(this, CHANNEL_ID)
                .setSmallIcon(R.drawable.ic_launcher_foreground)
                .setContentTitle("BLE GPS")
                .setContentText("BLE & Mock active")
                .setPriority(NotificationCompat.PRIORITY_DEFAULT)
                .setContentIntent(contentIntent)
                .setAutoCancel(false)
                .build()

            pBLE = BLEProvider(application, baseContext)
            mockGPS.start()
            startForeground(ServicesConstants.NOTIFICATION_ID.FOREGROUND_MOCK_SERVICE,notification)
        } else if(intent.action == ServicesConstants.MOCKSERVICEACTION.STOPFOREGROUND_ACTION) {
            Log.i(LOG_TAG, "Received Stop Foreground Intent ")
            pBLE.disconnectBLE()
            mockGPS.shutdown()
            stopForeground(true)
            stopSelf()
        } else if(intent.action == ServicesConstants.MOCKSERVICEACTION.TASKER_ACTION){
            Log.i(LOG_TAG, "intent action : ${intent.action}")
            if(this::pBLE.isInitialized)  {
                taskerChanged(intent.extras)
            }
        }
        return START_STICKY
    }
    override fun onDestroy() {
        super.onDestroy()
        Log.i(LOG_TAG, "In onDestroy")
    }

    override fun onBind(intent: Intent): IBinder? {
        Log.v(LOG_TAG, "in onBind")
        return binder
    }

    override fun onRebind(intent: Intent) {
        Log.v(LOG_TAG, "in onRebind")
        super.onRebind(intent)
    }

    override fun onUnbind(intent: Intent): Boolean {
        Log.v(LOG_TAG, "in onUnbind")
        return true
    }

    fun sendCalibrateBLE(calibrateval: Int)
    {
        val buffer = ByteBuffer.allocate(4)
        buffer.order(ByteOrder.LITTLE_ENDIAN)
        buffer.putInt(calibrateval)
        pBLE.sendCalibrate(buffer.array())
    }

   //fun sendConfBLE(bmpnrf: Boolean, bmpble: Boolean, bmp2gps: Boolean, exttemp: Boolean, gpsble: Boolean, gpsbleverb: Boolean, gpsserial: Boolean, gpsmode: Int, gpspoller: Short, gpssendbt: Short, bmppoller: Short) {
    fun sendConfBLE(mode: Int) {
        val buffer = ByteBuffer.allocate(2)
        buffer.order(ByteOrder.LITTLE_ENDIAN)
        var activateByte1: Byte = 0
        var activateByte2: Byte = 0

        if(sharedPreferences.getBoolean("swbmp2gps", false)) { activateByte1 = activateByte1.or(0x01) }
        if(mode == 1) {
            activateByte2 = activateByte2.or(0x01)
        } else if(mode == 2) {
            activateByte2 = activateByte2.or(0x02)
        } else if(mode == 3) {
            activateByte2 = activateByte2.or(0x04)
        }

        buffer.put(activateByte1)
        buffer.put(activateByte2)

        pBLE.sendConf(buffer.array())
    }

    inner class sBinder : Binder() {
        // Return this instance of LocalService so clients can call public methods
        fun getService(): BLEMockLocationService = this@BLEMockLocationService
    }
}